@@ -26,7 +26,8 @@ arch_native = pmb.parse.arch.alpine_native()
 # Update this frequently to prevent a MITM attack with an outdated version
 # (which may contain a vulnerable apk/openssl, and allows an attacker to
 # exploit the system!)
-apk_tools_min_version = {"edge": "2.14.0_rc1-r0",
+apk_tools_min_version = {"edge": "2.14.0-r5",
+                         "v3.18": "2.14.0-r2",
                          "v3.17": "2.12.10-r1",
                          "v3.16": "2.12.9-r3",
                          "v3.15": "2.12.7-r3",
@@ -60,32 +61,35 @@ required_programs = [
 sudo = which_sudo()
 
 # Keys saved in the config file (mostly what we ask in 'pmbootstrap init')
-config_keys = ["aports",
-               "ccache_size",
-               "device",
-               "extra_packages",
-               "hostname",
-               "build_pkgs_on_install",
-               "is_default_channel",
-               "jobs",
-               "kernel",
-               "keymap",
-               "locale",
-               "mirror_alpine",
-               "mirrors_postmarketos",
-               "nonfree_firmware",
-               "nonfree_userland",
-               "ssh_keys",
-               "ssh_key_glob",
-               "timezone",
-               "ui",
-               "ui_extras",
-               "user",
-               "work",
-               "boot_size",
-               "extra_space",
-               "sudo_timer",
-               "qemu_redir_stdio"]
+config_keys = [
+    "aports",
+    "boot_size",
+    "build_default_device_arch",
+    "build_pkgs_on_install",
+    "ccache_size",
+    "device",
+    "extra_packages",
+    "extra_space",
+    "hostname",
+    "is_default_channel",
+    "jobs",
+    "kernel",
+    "keymap",
+    "locale",
+    "mirror_alpine",
+    "mirrors_postmarketos",
+    "nonfree_firmware",
+    "nonfree_userland",
+    "qemu_redir_stdio",
+    "ssh_key_glob",
+    "ssh_keys",
+    "sudo_timer",
+    "timezone",
+    "ui",
+    "ui_extras",
+    "user",
+    "work",
+]
 
 # Config file/commandline default values
 # $WORK gets replaced with the actual value for args.work (which may be
@@ -128,7 +132,8 @@ defaults = {
     "boot_size": "256",
     "extra_space": "0",
     "sudo_timer": False,
-    "qemu_redir_stdio": False
+    "qemu_redir_stdio": False,
+    "build_default_device_arch": False,
 }
 
 
@@ -632,6 +637,20 @@ kconfig_options_filesystems = {
     },
 }
 
+kconfig_options_usb_gadgets = {
+    ">=0.0.0": {  # all versions
+        "all": {  # all arches
+            # disable legacy gadgets
+            "USB_ETH": False,
+            "USB_FUNCTIONFS": False,
+            "USB_MASS_STORAGE": False,
+            "USB_G_SERIAL": False,
+            # enable configfs gadgets
+            "USB_CONFIGFS_RNDIS": True,  # USB networking via RNDIS
+        },
+    },
+}
+
 # Various other kernel config options
 kconfig_options_community = {
     ">=0.0.0": {  # all versions
