@@ -1,4 +1,4 @@
-# Copyright 2022 Oliver Smith
+# Copyright 2023 Oliver Smith
 # SPDX-License-Identifier: GPL-3.0-or-later
 import glob
 import json
@@ -9,13 +9,14 @@ import sys
 import pmb.aportgen
 import pmb.build
 import pmb.build.autodetect
-import pmb.sideload
 import pmb.chroot
 import pmb.chroot.initfs
 import pmb.chroot.other
+import pmb.ci
 import pmb.config
 import pmb.export
 import pmb.flasher
+import pmb.helpers.aportupgrade
 import pmb.helpers.devices
 import pmb.helpers.git
 import pmb.helpers.lint
@@ -25,13 +26,13 @@ import pmb.helpers.pmaports
 import pmb.helpers.repo
 import pmb.helpers.repo_missing
 import pmb.helpers.run
-import pmb.helpers.aportupgrade
 import pmb.helpers.status
 import pmb.install
 import pmb.install.blockdevice
 import pmb.netboot
 import pmb.parse
 import pmb.qemu
+import pmb.sideload
 
 
 def _parse_flavor(args, autoinstall=True):
@@ -376,17 +377,17 @@ def newapkbuild(args):
 
 def kconfig(args):
     if args.action_kconfig == "check":
+        details = args.kconfig_check_details
+        # Build the components list from cli arguments (--waydroid etc.)
+        components_list = []
+        for name in pmb.parse.kconfig.get_all_component_names():
+            if getattr(args, f"kconfig_check_{name}"):
+                components_list += [name]
+
         # Handle passing a file directly
         if args.file:
-            if pmb.parse.kconfig.check_file(args.package,
-                                            waydroid=args.waydroid,
-                                            nftables=args.nftables,
-                                            containers=args.containers,
-                                            zram=args.zram,
-                                            netboot=args.netboot,
-                                            community=args.community,
-                                            uefi=args.uefi,
-                                            details=True):
+            if pmb.parse.kconfig.check_file(args.package, components_list,
+                                            details=details):
                 logging.info("kconfig check succeeded!")
                 return
             raise RuntimeError("kconfig check failed!")
@@ -413,17 +414,8 @@ def kconfig(args):
                 if "!pmb:kconfigcheck" in apkbuild["options"]:
                     skipped += 1
                     continue
-            if not pmb.parse.kconfig.check(
-                    args, package,
-                    force_waydroid_check=args.waydroid,
-                    force_iwd_check=args.iwd,
-                    force_nftables_check=args.nftables,
-                    force_containers_check=args.containers,
-                    force_zram_check=args.zram,
-                    force_netboot_check=args.netboot,
-                    force_community_check=args.community,
-                    force_uefi_check=args.uefi,
-                    details=True):
+            if not pmb.parse.kconfig.check(args, package, components_list,
+                                           details=details):
                 error = True
 
         # At least one failure
@@ -541,8 +533,21 @@ def work_migrate(args):
 def log(args):
     if args.clear_log:
         pmb.helpers.run.user(args, ["truncate", "-s", "0", args.log])
-    pmb.helpers.run.user(args, ["tail", "-n", args.lines, "-F", args.log],
-                         output="tui")
+
+    cmd = ["tail", "-n", args.lines, "-F"]
+
+    # Follow the testsuite's log file too if it exists. It will be created when
+    # starting a test case that writes to it (git -C test grep log_testsuite).
+    log_testsuite = f"{args.work}/log_testsuite.txt"
+    if os.path.exists(log_testsuite):
+        cmd += [log_testsuite]
+
+    # tail writes the last lines of the files to the terminal. Put the regular
+    # log at the end, so that output is visible at the bottom (where the user
+    # looks for an error / what's currently going on).
+    cmd += [args.log]
+
+    pmb.helpers.run.user(args, cmd, output="tui")
 
 
 def log_distccd(args):
@@ -610,3 +615,47 @@ def lint(args):
 def status(args):
     if not pmb.helpers.status.print_status(args, args.details):
         sys.exit(1)
+
+
+def ci(args):
+    topdir = pmb.helpers.git.get_topdir(args, os.getcwd())
+    if not os.path.exists(topdir):
+        logging.error("ERROR: change your current directory to a git"
+                      " repository (e.g. pmbootstrap, pmaports) before running"
+                      " 'pmbootstrap ci'.")
+        exit(1)
+
+    scripts_available = pmb.ci.get_ci_scripts(topdir)
+    scripts_available = pmb.ci.sort_scripts_by_speed(scripts_available)
+    if not scripts_available:
+        logging.error("ERROR: no supported CI scripts found in current git"
+                      " repository, see https://postmarketos.org/pmb-ci")
+        exit(1)
+
+    scripts_selected = {}
+    if args.scripts:
+        if args.all:
+            raise RuntimeError("Combining --all with script names doesn't"
+                               " make sense")
+        for script in args.scripts:
+            if script not in scripts_available:
+                logging.error(f"ERROR: script '{script}' not found in git"
+                              " repository, found these:"
+                              f" {', '.join(scripts_available.keys())}")
+                exit(1)
+            scripts_selected[script] = scripts_available[script]
+    elif args.all:
+        scripts_selected = scripts_available
+
+    if args.fast:
+        for script, script_data in scripts_available.items():
+            if "slow" not in script_data["options"]:
+                scripts_selected[script] = script_data
+
+    if not pmb.helpers.git.clean_worktree(args, topdir):
+        logging.warning("WARNING: this git repository has uncommitted changes")
+
+    if not scripts_selected:
+        scripts_selected = pmb.ci.ask_which_scripts_to_run(scripts_available)
+
+    pmb.ci.run_scripts(args, topdir, scripts_selected)
