@@ -1,4 +1,4 @@
-# Copyright 2022 Oliver Smith
+# Copyright 2023 Oliver Smith
 # SPDX-License-Identifier: GPL-3.0-or-later
 import argparse
 import copy
@@ -366,6 +366,8 @@ def arguments_qemu(subparser):
     ret.add_argument("--host-qemu", dest="host_qemu", action='store_true',
                      help="Use the host system's qemu")
 
+    ret.add_argument("--efi", action="store_true",
+                     help="Use EFI boot (default: direct kernel image boot)")
     return ret
 
 
@@ -464,23 +466,14 @@ def arguments_kconfig(subparser):
     check.add_argument("--arch", choices=arch_choices, dest="arch")
     check.add_argument("--file", action="store_true", help="check a file"
                        " directly instead of a config in a package")
-    check.add_argument("--waydroid", action="store_true", help="check"
-                       " options needed for waydroid too")
-    check.add_argument("--iwd", action="store_true", help="check"
-                       " options needed for iwd too")
-    check.add_argument("--nftables", action="store_true", help="check"
-                       " options needed for nftables too")
-    check.add_argument("--containers", action="store_true",
-                       help="check options needed for containers too")
-    check.add_argument("--zram", action="store_true", help="check"
-                       " options needed for zram support too")
-    check.add_argument("--netboot", action="store_true", help="check"
-                       " options needed for netbooting too")
-    check.add_argument("--community", action="store_true", help="check"
-                       " options needed for various features, required for"
-                       " community/main devices")
-    check.add_argument("--uefi", action="store_true", help="check"
-                       " options needed for uefi too")
+    check.add_argument("--no-details", action="store_false",
+                       dest="kconfig_check_details",
+                       help="print one generic error per component instead of"
+                            " listing each option that needs to be adjusted")
+    for name in pmb.parse.kconfig.get_all_component_names():
+        check.add_argument(f"--{name}", action="store_true",
+                           dest=f"kconfig_check_{name}",
+                           help=f"check options needed for {name} too")
     add_kernel_arg(check)
 
     # "pmbootstrap kconfig edit"
@@ -546,6 +539,21 @@ def arguments_netboot(subparser):
     return ret
 
 
+def arguments_ci(subparser):
+    ret = subparser.add_parser("ci", help="run continuous integration scripts"
+                                          " locally of git repo in current"
+                                          " directory")
+    script_args = ret.add_mutually_exclusive_group()
+    script_args.add_argument("-a", "--all", action="store_true",
+                             help="run all scripts")
+    script_args.add_argument("-f", "--fast", action="store_true",
+                             help="run fast scripts only")
+    ret.add_argument("scripts", nargs="*", metavar="script",
+                     help="name of the CI script to run, depending on the git"
+                          " repository")
+    return ret
+
+
 def package_completer(prefix, action, parser=None, parsed_args=None):
     args = parsed_args
     pmb.config.merge_with_args(args)
@@ -580,8 +588,8 @@ def add_packages_arg(subparser, name="packages", *args, **kwargs):
         arg.completer = package_completer
 
 
-def add_kernel_arg(subparser, name="package", *args, **kwargs):
-    arg = subparser.add_argument("package", nargs='?', help="kernel package"
+def add_kernel_arg(subparser, name="package", nargs="?", *args, **kwargs):
+    arg = subparser.add_argument("package", nargs=nargs, help="kernel package"
                                  " (e.g. linux-postmarketos-allwinner)")
     if argcomplete:
         arg.completer = kernel_completer
@@ -595,7 +603,7 @@ def arguments():
 
     # Other
     parser.add_argument("-V", "--version", action="version",
-                        version=pmb.config.version)
+                        version=pmb.__version__)
     parser.add_argument("-c", "--config", dest="config",
                         default=pmb.config.defaults["config"],
                         help="path to pmbootstrap.cfg file (default in"
@@ -693,6 +701,7 @@ def arguments():
     arguments_newapkbuild(sub)
     arguments_lint(sub)
     arguments_status(sub)
+    arguments_ci(sub)
 
     # Action: log
     log = sub.add_parser("log", help="follow the pmbootstrap logfile")
@@ -839,9 +848,21 @@ def arguments():
     build.add_argument("-n", "--no-depends", action="store_true",
                        help="never build dependencies, abort instead",
                        dest="no_depends")
+    build.add_argument("--go-mod-cache", action="store_true", default=None,
+                       help="for go packages: Usually they should bundle the"
+                            " dependency sources instead of downloading them"
+                            " at build time. But if they don't (e.g. with"
+                            " pmbootstrap build --src), then this option can"
+                            " be used to let GOMODCACHE point into"
+                            " pmbootstrap's work dir to only download"
+                            " dependencies once. (default: true with --src,"
+                            " false otherwise)")
+    build.add_argument("--no-go-mod-cache",
+                       action="store_false", dest="go_mod_cache", default=None,
+                       help="don't set GOMODCACHE")
     build.add_argument("--envkernel", action="store_true",
                        help="Create an apk package from the build output of"
-                       " a kernel compiled with envkernel.sh.")
+                       " a kernel compiled locally on the host or with envkernel.sh.")
     add_packages_arg(build, nargs="+")
 
     # Action: deviceinfo_parse
@@ -893,4 +914,9 @@ def arguments():
     setattr(args, "from_argparse", copy.deepcopy(args))
     setattr(args.from_argparse, "from_argparse", args.from_argparse)
     pmb.helpers.args.init(args)
+
+    if getattr(args, "go_mod_cache", None) is None:
+        gomodcache = True if getattr(args, "src", None) else False
+        setattr(args, "go_mod_cache", gomodcache)
+
     return args
